@@ -1,3 +1,5 @@
+//! Memory page functions
+
 use crate::auxiliary::align;
 use crate::traits::{AsPages, Protectable};
 
@@ -18,7 +20,20 @@ use std::ffi::c_void;
 #[cfg(windows)]
 use winapi::ctypes::c_void;
 
-#[cfg(unix)]
+#[cfg(doc)]
+/// Page access protection
+pub enum Protection {
+	/// Pages may not be accessed
+	NoAccess,
+
+	/// Pages may be read
+	ReadOnly,
+
+	/// Pages may be read and written
+	ReadWrite
+}
+
+#[cfg(all(unix, not(doc)))]
 #[repr(i32)]
 pub enum Protection {
 	NoAccess = libc::PROT_NONE,
@@ -26,7 +41,7 @@ pub enum Protection {
 	ReadWrite = libc::PROT_READ | libc::PROT_WRITE,
 }
 
-#[cfg(windows)]
+#[cfg(all(windows, not(doc)))]
 #[repr(u32)]
 pub enum Protection {
 	NoAccess = winnt::PAGE_NOACCESS,
@@ -34,12 +49,15 @@ pub enum Protection {
 	ReadWrite = winnt::PAGE_READWRITE,
 }
 
+/// Memory pages
 #[derive(Debug)]
 pub struct Pages<'t>(NonNull<[u8]>, PhantomData<&'t ()>);
 
+/// Memory page allocation
 #[derive(Debug)]
 pub struct Allocation(NonNull<[u8]>);
 
+/// Guarded memory page allocation
 #[derive(Debug)]
 pub struct GuardedAlloc<const N: usize = 1>(Allocation);
 
@@ -84,6 +102,11 @@ impl<'t> Pages<'t> {
 		align(offset, Self::granularity())
 	}
 
+	/// Create [`Pages`] from [non‐null](NonNull) raw [`u8`] slice
+	///
+	/// # Safety
+	///
+	/// `slice` must be aligned to and have a size that is a multiple of the [page size](Self::granularity).
 	pub unsafe fn from_slice(slice: NonNull<[u8]>) -> Pages<'t> {
 		// Assert correct alignment
 		debug_assert_eq!(slice.as_ptr().cast::<u8>().align_offset(Self::granularity()), 0);
@@ -92,15 +115,28 @@ impl<'t> Pages<'t> {
 		Self(slice, PhantomData)
 	}
 
+	/// Create [`Pages`] from [non‐null](NonNull) [`u8`] pointer and length
+	///
+	/// # Safety
+	///
+	/// `ptr` must be aligned to a multiple of the [page size](Self::granularity) and `size` must be a multiple of the
+	/// [page size](Self::granularity).
 	pub unsafe fn from_raw_parts(ptr: NonNull<u8>, size: usize) -> Pages<'t> {
 		Self::from_slice(NonNull::slice_from_raw_parts(ptr, size))
 	}
 
+	/// Create [`Pages`] from pointer and length
+	///
+	/// # Safety
+	///
+	/// `ptr` must be non‐null and aligned to a multiple of the [page size](Self::granularity) and `size` must be a
+	/// multiple of the [page size](Self::granularity).
 	pub unsafe fn from_ptr<T>(ptr: *mut T, size: usize) -> Pages<'t> {
 		Self::from_raw_parts(NonNull::new(ptr.cast::<u8>()).unwrap(), size)
 	}
 
-	pub unsafe fn as_ptr<T>(&self) -> *mut T {
+	pub fn as_ptr<T>(&self) -> *mut T {
+		debug_assert!(std::mem::align_of::<T>() < Self::granularity());
 		self.0.as_ptr().cast::<T>()
 	}
 
@@ -185,9 +221,8 @@ impl<'t> Pages<'t> {
 
 	pub fn pages(&'t self, range: Range<usize>) -> Option<Pages<'t>> {
 		if range.start < self.len() && range.end <= self.len() {
-			let ptr = unsafe { self.as_ptr::<u8>() };
 			Some(unsafe {
-				Self::from_ptr(ptr.add(range.start * Self::granularity()),
+				Self::from_ptr(self.as_ptr::<u8>().add(range.start * Self::granularity()),
 				(range.end - range.start - 1) * Self::granularity())
 			})
 		} else {
@@ -213,6 +248,12 @@ impl Allocation {
 		align(offset, Self::granularity())
 	}
 
+	/// Create [`Allocation`] from [non‐null](NonNull) raw [`u8`] slice
+	///
+	/// # Safety
+	///
+	/// `slice` must have been previously generated by a call to [`into_slice`](Self::into_slice) and must not be
+	/// aliased by any other [`Allocation`].
 	pub unsafe fn from_slice(slice: NonNull<[u8]>) -> Allocation {
 		// Assert correct alignment
 		debug_assert_eq!(slice.as_ptr().cast::<u8>().align_offset(Self::granularity()), 0);
@@ -221,20 +262,33 @@ impl Allocation {
 		Self(slice)
 	}
 
+	/// Create [`Allocation`] from [non‐null](NonNull) [`u8`] pointer and length
+	///
+	/// # Safety
+	///
+	/// `ptr` must have been previously generated by a call to [`into_ptr`](Self::into_ptr) and must not be aliased
+	/// by any other [`Allocation`]. `size` must match the size of the original `Allocation`.
 	pub unsafe fn from_raw_parts(ptr: NonNull<u8>, size: usize) -> Allocation {
 		Self::from_slice(NonNull::slice_from_raw_parts(ptr, size))
 	}
 
+	/// Create [`Allocation`] from pointer and length
+	///
+	/// # Safety
+	///
+	/// `ptr` must have been previously generated by a call to [`into_ptr`](Self::into_ptr) and must not be aliased
+	/// by any other [`Allocation`]. `size` must match the size of the original `Allocation`.
 	pub unsafe fn from_ptr<T>(ptr: *mut T, size: usize) -> Allocation {
 		Self::from_raw_parts(NonNull::new(ptr.cast::<u8>()).unwrap(), size)
 	}
 
-	pub unsafe fn as_ptr<T>(&self) -> *mut T {
+	pub fn as_ptr<T>(&self) -> *mut T {
+		debug_assert!(std::mem::align_of::<T>() < Self::granularity());
 		self.0.as_ptr().cast::<T>()
 	}
 
 	pub fn into_ptr<T>(self) -> *mut T {
-		unsafe { ManuallyDrop::new(self).as_ptr() }
+		ManuallyDrop::new(self).as_ptr()
 	}
 
 	pub fn into_slice(self) -> NonNull<[u8]> {
@@ -311,9 +365,8 @@ impl Allocation {
 
 	pub fn pages(&self, range: Range<usize>) -> Option<Pages> {
 		if range.start < self.len() && range.end <= self.len() {
-			let ptr = unsafe { self.as_ptr::<u8>() };
 			Some(unsafe {
-				Pages::from_ptr(ptr.add(range.start * Pages::granularity()),
+				Pages::from_ptr(self.as_ptr::<u8>().add(range.start * Pages::granularity()),
 				(range.end - range.start) * Pages::granularity())
 			})
 		} else {
@@ -372,6 +425,12 @@ impl<const N: usize> GuardedAlloc<N> {
 		self.0.pages(Self::GUARD_PAGES .. self.0.len() - Self::GUARD_PAGES).unwrap()
 	}
 
+	/// Create [`GuardedAlloc`] from [non‐null](NonNull) raw [`u8`] slice
+	///
+	/// # Safety
+	///
+	/// `slice` must have been previously generated by a call to [`into_slice`](Self::into_slice) and must not be
+	/// aliased by any other [`GuardedAlloc`].
 	pub unsafe fn from_raw_parts(base: NonNull<u8>, inner: usize) -> Self {
 		debug_assert_eq!(base.as_ptr().align_offset(Pages::granularity()), 0);
 
@@ -384,6 +443,12 @@ impl<const N: usize> GuardedAlloc<N> {
 		Self(Allocation::from_ptr(ptr, outer))
 	}
 
+	/// Create [`GuardedAlloc`] from pointer and length
+	///
+	/// # Safety
+	///
+	/// `ptr` must have been previously generated by a call to [`into_slice`](Self::into_slice) and must not be
+	/// aliased by any other [`GuardedAlloc`]. `size` must match the size of the original [`GuardedAlloc`].
 	pub unsafe fn from_ptr<T>(base: *mut T, inner: usize) -> Self {
 		Self::from_raw_parts(NonNull::new(base.cast::<u8>()).unwrap(), inner)
 	}
@@ -519,7 +584,7 @@ mod tests {
 		let size = Allocation::granularity();
 		let bp = unsafe { Bulletproof::new() };
 		let alloc = Allocation::new(size, Protection::NoAccess).unwrap();
-		let ptr = unsafe { alloc.as_ptr::<u8>() };
+		let ptr = alloc.as_ptr::<u8>();
 
 		for i in 0..size {
 			assert_eq!(unsafe { bp.load(ptr.add(i)) }, Err(()));
@@ -552,7 +617,7 @@ mod tests {
 		let alloc_0 = Allocation::new(size_0, Protection::ReadWrite).unwrap();
 		assert_eq!(alloc_0.size(), size_0);
 
-		let ptr = unsafe { alloc_0.as_ptr::<u8>() };
+		let ptr = alloc_0.as_ptr::<u8>();
 
 		for i in 0..size_0 {
 			assert_eq!(unsafe { bp.load(ptr.add(i)) }, Ok(0));
@@ -636,7 +701,7 @@ mod tests {
 		let size = Allocation::granularity();
 		let bp = unsafe { Bulletproof::new() };
 		let alloc = GuardedAlloc::<1>::new(size, Protection::ReadWrite).unwrap();
-		let ptr = unsafe { alloc.inner().as_ptr::<u8>() };
+		let ptr = alloc.inner().as_ptr::<u8>();
 
 		// Preceding guard
 		for i in 1 ..= GuardedAlloc::<1>::guard_size() {
@@ -665,7 +730,7 @@ mod tests {
 
 		let bp = unsafe { Bulletproof::new() };
 		let alloc_0 = GuardedAlloc::<1>::new(size_0, Protection::ReadWrite).unwrap();
-		let ptr = unsafe { alloc_0.inner().as_ptr::<u8>() };
+		let ptr = alloc_0.inner().as_ptr::<u8>();
 
 		for i in 0..size_0 {
 			assert_eq!(unsafe { bp.load(ptr.add(i)) }, Ok(0));
@@ -680,7 +745,7 @@ mod tests {
 		let alloc_1 = alloc_0.shrink(size_1).unwrap();
 
 		// Allocation should not move
-		assert_eq!(unsafe { alloc_1.inner().as_ptr::<u8>() }, ptr);
+		assert_eq!(alloc_1.inner().as_ptr::<u8>(), ptr);
 
 		for i in 0 .. size_1 {
 			assert_eq!(unsafe { bp.load(ptr.add(i)) }, Ok(0));
