@@ -1,3 +1,5 @@
+//! Guarded [vector](mod@std::vec) type
+
 use crate::auxiliary::zero;
 use crate::pages::{Pages, GuardedAlloc};
 use crate::alloc::Sensitive;
@@ -9,6 +11,8 @@ use std::default::Default;
 use std::mem::MaybeUninit;
 
 pub(crate) type InnerVec<T> = std::vec::Vec<T, Sensitive>;
+
+/// Guarded [vector](std::vec::Vec) type
 pub type Vec<T> = Guard<InnerVec<T>>;
 
 impl<T> AsPages for InnerVec<T> {
@@ -85,11 +89,11 @@ impl<T> Vec<T> {
 	}
 
 	pub fn as_ptr(&self) -> *const T {
-		unsafe { self.inner().as_ptr() }
+		unsafe { self.inner() }.as_ptr()
 	}
 
 	pub fn as_mut_ptr(&mut self) -> *mut T {
-		unsafe { self.inner_mut().as_mut_ptr() }
+		unsafe { self.inner_mut() }.as_mut_ptr()
 	}
 }
 
